@@ -1,6 +1,13 @@
 use crate::{utils::*, Signature};
 use serde::de::{Deserialize, DeserializeSeed};
-use std::{convert::TryInto, marker::PhantomData};
+use std::{
+    marker::PhantomData,
+    net::{IpAddr, Ipv4Addr, Ipv6Addr},
+    path::{Path, PathBuf},
+    rc::Rc,
+    sync::{Arc, Mutex, RwLock},
+    time::Duration,
+};
 
 /// Trait implemented by all serializable types.
 ///
@@ -23,7 +30,7 @@ use std::{convert::TryInto, marker::PhantomData};
 /// [basic types]: trait.Basic.html
 /// [`Vec`]: https://doc.rust-lang.org/std/vec/struct.Vec.html
 /// [`HashMap`]: https://doc.rust-lang.org/std/collections/struct.HashMap.html
-/// [zvariant_derive]: https://docs.rs/zvariant_derive/2.0.0/zvariant_derive/
+/// [zvariant_derive]: https://docs.rs/zvariant_derive/latest/zvariant_derive/
 pub trait Type {
     /// Get the signature for the implementing type.
     ///
@@ -106,7 +113,7 @@ where
             return Ok(PhantomData);
         }
 
-        let mut signature = original.clone();
+        let mut signature = original.as_ref();
         while expected.len() < signature.len()
             && signature.starts_with(STRUCT_SIG_START_CHAR)
             && signature.ends_with(STRUCT_SIG_END_CHAR)
@@ -127,7 +134,7 @@ where
             let expected = <T as Type>::signature();
             Err(zvariant::Error::SignatureMismatch(
                 original.to_owned(),
-                format!("`{}`", expected),
+                format!("`{expected}`"),
             ))
         }
     }
@@ -150,10 +157,20 @@ macro_rules! array_type {
 array_type!([T]);
 array_type!(Vec<T>);
 
+impl<T, S> Type for std::collections::HashSet<T, S>
+where
+    T: Type + Eq + Hash,
+    S: BuildHasher,
+{
+    #[inline]
+    fn signature() -> Signature<'static> {
+        <[T]>::signature()
+    }
+}
+
 #[cfg(feature = "arrayvec")]
-impl<A, T> Type for arrayvec::ArrayVec<A>
+impl<T, const CAP: usize> Type for arrayvec::ArrayVec<T, CAP>
 where
-    A: arrayvec::Array<Item = T>,
     T: Type,
 {
     #[inline]
@@ -163,10 +180,7 @@ where
 }
 
 #[cfg(feature = "arrayvec")]
-impl<A> Type for arrayvec::ArrayString<A>
-where
-    A: arrayvec::Array<Item = u8> + Copy,
-{
+impl<const CAP: usize> Type for arrayvec::ArrayString<CAP> {
     #[inline]
     fn signature() -> Signature<'static> {
         <&str>::signature()
@@ -181,17 +195,30 @@ impl Type for () {
     }
 }
 
-impl<T> Type for &T
-where
-    T: ?Sized + Type,
-{
-    #[inline]
-    fn signature() -> Signature<'static> {
-        T::signature()
-    }
+macro_rules! deref_impl {
+    (
+        $type:ty,
+        <$($desc:tt)+
+    ) => {
+        impl <$($desc)+ {
+            #[inline]
+            fn signature() -> Signature<'static> {
+                <$type>::signature()
+            }
+        }
+    };
 }
 
-#[cfg(feature = "gvariant")]
+deref_impl!(T, <T: ?Sized + Type> Type for &T);
+deref_impl!(T, <T: ?Sized + Type> Type for &mut T);
+deref_impl!(T, <T: ?Sized + Type + ToOwned> Type for Cow<'_, T>);
+deref_impl!(T, <T: ?Sized + Type> Type for Arc<T>);
+deref_impl!(T, <T: ?Sized + Type> Type for Mutex<T>);
+deref_impl!(T, <T: ?Sized + Type> Type for RwLock<T>);
+deref_impl!(T, <T: ?Sized + Type> Type for Box<T>);
+deref_impl!(T, <T: ?Sized + Type> Type for Rc<T>);
+
+#[cfg(all(feature = "gvariant", not(feature = "option-as-array")))]
 impl<T> Type for Option<T>
 where
     T: Type,
@@ -202,6 +229,17 @@ where
     }
 }
 
+#[cfg(feature = "option-as-array")]
+impl<T> Type for Option<T>
+where
+    T: Type,
+{
+    #[inline]
+    fn signature() -> Signature<'static> {
+        Signature::from_string_unchecked(format!("a{}", T::signature()))
+    }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 macro_rules! tuple_impls {
@@ -211,7 +249,6 @@ macro_rules! tuple_impls {
             where
                 $($name: Type,)+
             {
-                #[inline]
                 fn signature() -> Signature<'static> {
                     let mut sig = String::with_capacity(255);
                     sig.push(STRUCT_SIG_START_CHAR);
@@ -251,44 +288,30 @@ tuple_impls! {
 // Arrays are serialized as tuples/structs by Serde so we treat them as such too even though
 // it's very strange. Slices and arrayvec::ArrayVec can be used anyway so I guess it's no big
 // deal.
-// TODO: Mention this fact in the module docs.
+impl<T, const N: usize> Type for [T; N]
+where
+    T: Type,
+{
+    #[allow(clippy::reversed_empty_ranges)]
+    fn signature() -> Signature<'static> {
+        let mut sig = String::with_capacity(255);
+        sig.push(STRUCT_SIG_START_CHAR);
+        for _ in 0..N {
+            sig.push_str(T::signature().as_str());
+        }
+        sig.push(STRUCT_SIG_END_CHAR);
 
-macro_rules! array_impls {
-    ($($len:tt)+) => {
-        $(
-            impl<T> Type for [T; $len]
-            where
-                T: Type,
-            {
-                #[inline]
-                #[allow(clippy::reversed_empty_ranges)]
-                fn signature() -> Signature<'static> {
-                    let mut sig = String::with_capacity(255);
-                    sig.push(STRUCT_SIG_START_CHAR);
-                    if $len > 0 {
-                        for _ in 0..$len {
-                            sig.push_str(T::signature().as_str());
-                        }
-                    }
-                    sig.push(STRUCT_SIG_END_CHAR);
-
-                    Signature::from_string_unchecked(sig)
-                }
-            }
-        )+
+        Signature::from_string_unchecked(sig)
     }
 }
 
-array_impls! {
-    0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 
 use std::{
     borrow::Cow,
     collections::{BTreeMap, HashMap},
     hash::{BuildHasher, Hash},
+    time::SystemTime,
 };
 
 macro_rules! map_impl {
@@ -310,13 +333,42 @@ macro_rules! map_impl {
 map_impl!(BTreeMap<K: Ord, V>);
 map_impl!(HashMap<K: Eq + Hash, V, H: BuildHasher>);
 
-impl<T> Type for Cow<'_, T>
-where
-    T: ?Sized + Type + ToOwned,
-{
+impl Type for Duration {
+    fn signature() -> Signature<'static> {
+        <(u64, u32)>::signature()
+    }
+}
+
+impl Type for SystemTime {
     #[inline]
     fn signature() -> Signature<'static> {
-        T::signature()
+        <(
+            // seconds
+            u64,
+            // nano
+            u32,
+        )>::signature()
+    }
+}
+
+impl Type for Ipv4Addr {
+    #[inline]
+    fn signature() -> Signature<'static> {
+        <[u8; 4]>::signature()
+    }
+}
+
+impl Type for Ipv6Addr {
+    #[inline]
+    fn signature() -> Signature<'static> {
+        <[u8; 16]>::signature()
+    }
+}
+
+impl Type for IpAddr {
+    #[inline]
+    fn signature() -> Signature<'static> {
+        <(u32, &[u8])>::signature()
     }
 }
 
@@ -324,7 +376,7 @@ where
 #[cfg(feature = "enumflags2")]
 impl<F> Type for enumflags2::BitFlags<F>
 where
-    F: Type + enumflags2::RawBitFlags,
+    F: Type + enumflags2::BitFlag,
 {
     #[inline]
     fn signature() -> Signature<'static> {
@@ -346,4 +398,155 @@ impl Type for serde_bytes::ByteBuf {
     }
 }
 
+#[allow(unused)]
+macro_rules! static_str_type {
+    ($ty:ty) => {
+        impl Type for $ty {
+            fn signature() -> Signature<'static> {
+                <&str>::signature()
+            }
+        }
+    };
+}
+
+static_str_type!(Path);
+static_str_type!(PathBuf);
+
+#[cfg(feature = "uuid")]
+impl Type for uuid::Uuid {
+    fn signature() -> Signature<'static> {
+        Signature::from_static_str_unchecked("ay")
+    }
+}
+
+#[cfg(feature = "url")]
+static_str_type!(url::Url);
+
+// FIXME: Ignoring the `serde-human-readable` feature of `time` crate in these impls:
+// https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L110
+#[cfg(feature = "time")]
+impl Type for time::Date {
+    fn signature() -> Signature<'static> {
+        // Serialized as a (year, ordinal) tuple:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L92
+        <(i32, u16)>::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::Duration {
+    fn signature() -> Signature<'static> {
+        // Serialized as a (whole seconds, nanoseconds) tuple:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L119
+        <(i64, i32)>::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::OffsetDateTime {
+    fn signature() -> Signature<'static> {
+        // Serialized as a tuple:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L155
+        <(
+            // year
+            i32,
+            // ordinal
+            u16,
+            // hour
+            u8,
+            // minute
+            u8,
+            // second
+            u8,
+            // nanosecond
+            u32,
+            // offset.whole_hours
+            i8,
+            // offset.minutes_past_hour
+            i8,
+            // offset.seconds_past_minute
+            i8,
+        )>::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::PrimitiveDateTime {
+    fn signature() -> Signature<'static> {
+        // Serialized as a tuple:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L200
+        <(
+            // year
+            i32,
+            // ordinal
+            u16,
+            // hour
+            u8,
+            // minute
+            u8,
+            // second
+            u8,
+            // nanosecond
+            u32,
+        )>::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::Time {
+    fn signature() -> Signature<'static> {
+        // Serialized as a tuple:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L246
+        <(
+            // hour
+            u8,
+            // minute
+            u8,
+            // second
+            u8,
+            // nanosecond
+            u32,
+        )>::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::UtcOffset {
+    fn signature() -> Signature<'static> {
+        // Serialized as a (whole hours, minutes past hour, seconds past minute) tuple:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L282
+        <(i8, i8, i8)>::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::Weekday {
+    fn signature() -> Signature<'static> {
+        // Serialized as number from Monday:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L312
+        u8::signature()
+    }
+}
+
+#[cfg(feature = "time")]
+impl Type for time::Month {
+    fn signature() -> Signature<'static> {
+        // Serialized as month number:
+        // https://github.com/time-rs/time/blob/f9398b9598757508ca3815694f23203843e0011b/src/serde/mod.rs#L337
+        u8::signature()
+    }
+}
+
+#[cfg(feature = "chrono")]
+impl<Tz: chrono::TimeZone> Type for chrono::DateTime<Tz> {
+    fn signature() -> Signature<'static> {
+        <&str>::signature()
+    }
+}
+
+#[cfg(feature = "chrono")]
+static_str_type!(chrono::NaiveDateTime);
+#[cfg(feature = "chrono")]
+static_str_type!(chrono::NaiveTime);
+
 // TODO: Blanket implementation for more types: https://github.com/serde-rs/serde/blob/master/serde/src/ser/impls.rs
